#!/usr/bin/perl
#
# Adrian Sun <asun@cobaltnet.com>
# Copyright 1999 Cobalt Networks http://www.cobaltnet.com/
#
# inspired by siteUserList.cgi

use Cobalt::Product;
use Cobalt::Vsite;
use Cobalt::DiskUtils;
use Cobalt::Language;
use Cobalt::Util;
use Cobalt::Locale;
use CGI;

use Locale::gettext;
Locale::gettext::textdomain("ui");

my $info;

my $q = new CGI;
my $page = $q->param('page');

if( $ENV{ PARAM_INFO } ) {
    $info=$ENV{ PARAM_INFO };
} else {
    $info=$q->param( 'info' );
}

my $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); }

my $KBperGB = 1024*1024;
my $diskListPageURL ="/cgi-bin/.cobalt/disk/diskList.cgi";
my $diskAddPageURL ="/cgi-bin/.cobalt/disk/diskAdd.cgi";
my $diskadd_msg = MSG_get_ui("volDiskAdd");
my $disabled_msg = '<IMG SRC=/.cobalt/images/suspend_icon_small.gif ALT="no">';
my $enabled_msg = '<IMG SRC=/.cobalt/images/yes.gif ALT="yes">';

my $error_voldelete = MSG_get_ui("volErrDeleteBusy");

my $str_help = MSG_get_ui("help");
my $str_volModify = MSG_get_ui("volModifyHelp");
my $str_volDelete = MSG_get_ui("volDeleteHelp");
my $ide_msg = MSG_get_ui("IDE");
my $scsi_msg = MSG_get_ui("SCSI");
my $raid_msg = MSG_get_ui("RAID");
my $unknown_msg = MSG_get_ui("volErrUnknown");

if ($page eq 'diskList') {
    $info = handleDiskListPage($q);
}
print "content-type: text/html\n";
print "Cache-Control: no-cache\n\n";
printDiskList( \*STDOUT, $info );
1;


sub dev_to_name
{
    my $dev = shift;
    
    if ($dev =~ /^\/dev\/h/) {
	return $ide_msg;
    }
    if ($dev =~ /^\/dev\/s/) {
	return $scsi_msg;
    }
    if ($dev =~ /^\/dev\/m/) {
	return $raid_msg;
    }

    return $unknown_msg;
}

# this just handles disk deletion
sub handleDiskListPage
{
    my $q = shift;
    my $label = $q->param('label');
    my $name = "$Vol_rootdir/$label";

    return undef unless defined($label) and -e $name;
    if (-f $name) {
	unlink($name);
	return undef;
    } elsif (-d $name) {
	my $link;

	opendir(DIR, $Htgroup_dir);
	while ($_ = readdir(DIR)) {
	    next unless $_ =~ /^site/; 
	    next unless (-l "$Htgroup_dir/$_");
	    $link = readlink("$Htgroup_dir/$_");
	    next unless $link =~ /^$name\/sites/o;
	    site_del($_);
	}
	closedir(DIR);

	# loop through again to see if something wrong happened.
	opendir(DIR, $Htgroup_dir);
	while ($_ = readdir(DIR)) {
	    next unless $_ =~ /^site/; 
	    next unless (-l "$Htgroup_dir/$_");
	    $link = readlink("$Htgroup_dir/$_");
	    next unless $link =~ /^$name\/sites/o;
	    closedir(DIR);
	    return $error_voldelete;
	}
	closedir(DIR);

	# just wipe out the disk label
	if (disk_getmounts($name)) {
	    unlink("$name/.cobaltsig");
	    return $error_voldelete if disk_umount($name);
	}
	unlink("$name/.cobaltsig");  # just in case of a screwup
	return $error_voldelete unless rmdir($name);
    }
    return undef;
}


# construct a form for doing stuff to a disk
sub constructForm
{
    my ($device, $label, $state) = @_;

    my $text = <<END;
<A HREF="javascript: modifyDisk(\'$device\', \'$label\', \'$state\');" onMouseOver="status='$str_help'; top.showInfo(modifyHelp); return true;"><IMG SRC="/.cobalt/images/modify.gif" ALT="$str_volModify" BORDER="0"></A><FONT COLOR="#FFFFFF">&nbsp;</FONT><A HREF="javascript: deleteDisk(\'$label\');" onMouseOver="status='$str_help'; top.showInfo(trashHelp); return true;"><IMG SRC="/.cobalt/images/delete.gif" ALT="$str_volDelete" BORDER="0"></A>
END

    return $text;
}

sub printDiskList
# description: print out a list of all the disks in use.
#
{
    my ($handle, $info) = @_;
    my (%mounts, %hash);
    my ($disklist, $key);
    my ($interface, $size, $label, $misc);
    my ($hidestart, $hideend) = ("<!-- ", " -->");
    my $directories = "/.//../";

    # do /home first.
    %mounts = disk_getmounts("/$Vol_home");
    foreach $key (sort keys %mounts) {
	$label = $Vol_home;
	$interface = dev_to_name($key);
	$size = sprintf("%.3f GB", $mounts{$key}{Size}/$KBperGB); 
	$disklist .= <<END;
<TR>
<TD BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT>$label</TD>
<TD ALIGN=CENTER BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT>$interface</TD>
<TD ALIGN=RIGHT BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT>$size</TD>
<TD ALIGN=CENTER BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT>$enabled_msg</TD>
<TD BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT><IMG SRC="/.cobalt/images/modifyGray.gif" BORDER="0"><FONT COLOR="#FFFFFF">&nbsp;</FONT><IMG SRC="/.cobalt/images/deleteGray.gif" BORDER="0"></TD>
END
    }
	
    # do /vol next. 
    %mounts = disk_getmounts($Vol_rootdir);
    foreach $key (keys %mounts) {
	$label = disk_getlabel($mounts{$key}{MountPoint});
	if ($label) {
	    $hash{$label}{Interface} = dev_to_name($key);
	    $hash{$label}{Size} = 
		sprintf("%.3f GB", $mounts{$key}{Size}/$KBperGB);
	    $hash{$label}{Misc} = constructForm($key, $label, "enabled"); 
	    $directories .= "/$label/";
	}
    }

    foreach $key (sort keys %hash) {
	$disklist .= <<END;
<TR>
<TD BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT>$key</TD>
<TD ALIGN=CENTER BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT>$hash{$key}{Interface}</TD>
<TD ALIGN=RIGHT BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT>$hash{$key}{Size}</TD>
<TD ALIGN=CENTER BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT>$enabled_msg</TD>
<TD BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT>$hash{$key}{Misc}</TD>
</TR>
END
    }

    # get all of the disabled disks. usually, they're disabled because
    # the enabled option is set to no, but somebody might have come
    # around and just unmounted a disk as well. 
    # NOTE: we need to disable this bit when a disk update is in progress.
    if (disk_checklock) {
	$hidestart = undef;
	$hideend = undef;
	$disklist .= <<END;
<TR>
<TD COLSPAN=5 BGCOLOR="#FFFFFF" ALIGN=CENTER><FONT COLOR=#FFFFFF>&nbsp;</FONT>$diskadd_msg</TD>
</TR>
END
    } else {
	%mounts = disk_getumounts;
	%hash = ();
	foreach $key (keys %mounts) {
	    my $mountpoint;
	    
	    if ($mountpoint = disk_mount($key, "ro,nocheck")) {
		my ($id, $enabled);

		($label, $id, $enabled) = disk_getsignature($mountpoint);
		next unless $label;
		$hash{$label}{Interface} = dev_to_name($key);
		$hash{$label}{Size} = 
		    sprintf("%.3f GB", $mounts{$key}/$KBperGB);
		$hash{$label}{Misc} = 
		    constructForm($key, $label, "disabled");
		$directories .= "/$label/";
		disk_umount($mountpoint);
	    }
	}

	foreach $key (sort keys %hash) {
	    $disklist .= <<END;
<TR>
<TD BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT><FONT COLOR=#555555>$key</FONT></TD>
<TD ALIGN=CENTER BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT><FONT COLOR=#555555>$hash{$key}{Interface}</FONT></TD>
<TD ALIGN=RIGHT BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT><FONT COLOR=#555555>$hash{$key}{Size}</FONT></TD>
<TD ALIGN=CENTER BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT><FONT COLOR=#555555>$disabled_msg</FONT></TD>
<TD BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT><FONT COLOR=#555555>$hash{$key}{Misc}</FONT></TD>
</TR>
END
        }


	# now get all of the directories without disks
	if (opendir(ROOTDIR, $Vol_rootdir)) {
	    while ($key = readdir(ROOTDIR)) {
		if ($directories !~ /\/$key\//) {
		    $disklist .= <<END;
<TR>
<TD BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT><FONT COLOR=#AAAAAA>$key</FONT></TD>
<TD ALIGN=CENTER BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT><FONT COLOR=#AAAAAA>$unknown_msg</FONT></TD>
<TD ALIGN=RIGHT BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT><FONT COLOR=#AAAAAA>$unknown_msg</FONT></TD>
<TD ALIGN=CENTER BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT><FONT COLOR=#AAAAAA>$disabled_msg</FONT></TD>
<TD BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT><IMG SRC="/.cobalt/images/modifyGray.gif" BORDER="0"><FONT COLOR=#FFFFFF>&nbsp;</FONT><A HREF="javascript: deleteDisk(\'$key\');" onMouseOver="status='$str_help'; top.showInfo(trashHelp); return true;"><IMG SRC="/.cobalt/images/delete.gif" ALT="$str_volDelete" BORDER="0"></A></TD>
</TR>
END
                }
	    }
	    closedir(ROOTDIR);
	}
    }

    open(TEMPLATE, "$Template_dir/diskList.tmpl") or 
	die "Can't read $Template_dir/diskList.tmpl: $!\n";

    while (<TEMPLATE>) {
	s/\[DISK_LIST\]/$disklist/o;
	s/\[DISK_INFO\]/$info/o;
	s/\[HIDESTART\]/$hidestart/o;
	s/\[HIDEEND\]/$hideend/o;
	s/\[DISKLISTPAGEURL\]/$diskListPageURL/o;
	s/\[DISKADDPAGEURL\]/$diskAddPageURL/o;
	s/\[str.([^\]]+)\]/MSG_get_ui("$1")/geo;
	print $handle $_;
    }
    close(TEMPLATE);

}

