#!/usr/bin/perl

# author: Kevin K.M. Chiu
#       Cobalt Microserver
# version: 1.0
# date: 7-9-98
#
# Copyright 1998-1999, Cobalt Networks, Inc
# All rights reserved

use Cobalt::Product;
use Cobalt::Locale;
use Cobalt::Netutil;
use Cobalt::Util;
use Cobalt::Meta;
use Cobalt::BandWidth;

use DB_File;
use Fcntl qw(O_RDWR O_CREAT);
use CGI;
use strict;
use Locale::gettext;

# global variables
my $siteDefaultsScriptURL='/cgi-bin/.cobalt/siteDefaults/siteDefaults.cgi';
my $siteListScriptURL='/cgi-bin/.cobalt/siteList/siteList.cgi';

my $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); }

my $q=new CGI;
my $page=$q->param( 'page' );

if ($q->param('error') eq 'yes') {
    my $errobj = Cobalt::Meta->new("type" => "vsite",
				   "name" => "default",
				   "hostname" => "",
				   "domain" => $q->param('domain'),
				   "ipaddr" => $q->param('ipaddr'),
				   "fpx" => $q->param('fpx'),
				   "maxusers" => $q->param('maxusers'),
				   "quota" => $q->param('quota'),
				   "emaildomain" => $q->param('emaildomain'),
				   "webdomain" => $q->param('webdomain'),
				   "shell" => $q->param('shell'),
				   "bwlimit" => $q->param('bwlimit'),
				   "cgi" => $q->param('cgi'),
				   "ssl" => $q->param('ssl'),
				   "ssi" => $q->param('ssi'),
				   "apop" => $q->param('apop'),
				   "ftp" => $q->param('ftp'),
				   "ftpusers" => $q->param('ftpusers'),
				   "ftpquota" => $q->param('ftpquota'));

    my $errbwobj = Cobalt::Meta->new("type" => "bw");
    $errbwobj->put("bwlimit" => $q->param('bw_setip'));
    
    genSiteDefaultsPage($errobj, $errbwobj, $q->param('info'));

} elsif( $page ne 'sitedefaults' ) {

    my $defobj = Cobalt::Meta->new("type" => "vsite");
    $defobj->retrieve("default");
    my $defbwobj = Cobalt::Meta->new("type" => "bw");
    $defbwobj->retrieve("default");
    genSiteDefaultsPage($defobj, $defbwobj);

} else {

    my $cgi=$q->param('cgi');
    my $shell=$q->param('shell');
    my $ssi=$q->param('ssi');
    my $ssl=$q->param('ssl');
    my $bwlimit=$q->param('bwlimit');
    my $bw_setip=$q->param('bw_setip');
    my $ipaddr=$q->param('ipaddr');
    my $fpx=$q->param('fpx');
    my $domain=$q->param('domain');
    my $emaildomain=$q->param('emaildomain');
    my $webdomain=$q->param('webdomain');
    my $maxusers=$q->param('maxusers');
    my $diskQuota=$q->param('diskquota');
    my $ftp=$q->param('ftp');
    my $apop=$q->param('apop');
    my $ftpQuota=$q->param('ftpquota');
    my $ftpUsers=$q->param('ftpusers');

    $fpx ? ($fpx = 'on') : ($fpx = 'off');
    $ftp ? ($ftp = 'on') : ($ftp = 'off');
    $cgi ? ($cgi = 'on') : ($cgi = 'off');
    $ssl ? ($ssl = 'on') : ($ssl = 'off');
    $ssi ? ($ssi = 'on') : ($ssi = 'off');
    $bwlimit ? ($bwlimit = 'on') : ($bwlimit = 'off');
    $apop ? ($apop = 'on') : ($apop = 'off');
    $shell ? ($shell = 'on') : ($shell = 'off');
    $emaildomain ? ($emaildomain = 'on') : ($emaildomain = 'off');
    $webdomain ? ($webdomain = 'on') : ($webdomain = 'off');

    my $maxQuotaInMB=$MaxQuota*$BytesPerBlock/1048576;

    my $info='<IMG SRC="/.cobalt/images/bang.gif" ALIGN=CENTER>';
    my $err=0;

    # check disk quota
    if( $diskQuota!~/^\d+$/o || $diskQuota==0 ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "diskQuotaNotPositive")."\"") . "<BR>";
	$err=1;
    }
    elsif( $diskQuota>$maxQuotaInMB ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "diskQuotaTooBig")."\"") . "<BR>";
	$err=1;
    }

    if( $ftpQuota!~/^\d+$/o ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "ftpQuotaNotInteger")."\"") . "<BR>";
	$err=1;
    }
    elsif( $ftpQuota>$diskQuota ) {
	my $siteDiskQuota=$diskQuota;
	$info.=eval("\"".Locale::gettext::dgettext("ui", "ftpQuotaTooBig")."\"") . "<BR>";
	$err=1;
    }

    if( $ftpUsers!~/^\d+$/o || $ftpUsers==0 ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "ftpUsersNotPositive")."\"") . "<BR>";
	$err=1;
    }

    if ( $maxusers !~ /^\d*$/ || $maxusers <= 0 ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "maxUsersNotValid")."\"") . "<BR>";
	$err=1;
    }

    if (! netutil_test_domainname($domain) && $domain) {
	my($domainName) = $domain; # Match embedded variable
	$info.=eval("\"".Locale::gettext::dgettext("ui", "domainNameNotValid")."\"") . "<BR>";
	$err=1;
    }

    if (!netutil_test_ipaddr($ipaddr) && $ipaddr) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "addrNotValid")."\"") . "<BR>" . "<BR>";
	$err=1;
    }

    # save all vsite info in the database
    my $obj = Cobalt::Meta->new("type" => "vsite");
    $obj->retrieve("default");

    my $o_user_quota = $obj->get("user_quota");
    my $o_user_fpx = $obj->get("user_fpx");
    my $o_user_shell = $obj->get("user_shell");
    my $o_user_apop = $obj->get("user_apop");
    my $o_user_namegen = $obj->get("user_namegen");
    my $o_userlist_sort = $obj->get("userlist_sort");
    my $o_userlist_range = $obj->get("userlist_range");

    my $default = Cobalt::Meta->new("type" => "vsite",
				    "name" => "default",
				    "hostname" => "",
				    "domain" => $domain,
				    "ipaddr" => $ipaddr,
				    "fpx" => $fpx,
				    "maxusers" => $maxusers,
				    "quota" => $diskQuota,
				    "emaildomain" => $emaildomain,
				    "webdomain" => $webdomain,
				    "shell" => $shell,
				    "bwlimit" => $bwlimit,
				    "cgi" => $cgi,
				    "ssl" => $ssl,
				    "ssi" => $ssi,
				    "apop" => $apop,
				    "suspend" => "no",
				    "ftp" => $ftp,
				    "ftpusers" => $ftpUsers,
				    "ftpquota" => $ftpQuota,
				    "user_quota" => $o_user_quota,
				    "user_fpx" => $o_user_fpx,
				    "user_shell" => $o_user_shell,
				    "user_apop" => $o_user_apop,
				    "user_namegen" => $o_user_namegen,
				    "userlist_sort" => $o_userlist_sort,
				    "userlist_range" => $o_userlist_range);

    my $defbw = Cobalt::Meta->new("type" => "bw",
				  "name" => "default",
				  "method" => "ip",
				  "bwlimit" => $bw_setip,
				  "protocol" => "tcp",
				  "port" => "",
				  "direction" => "w");
    if( !$err ) {
	$default->save;
	$defbw->save;

	print "location: $siteListScriptURL\n\n";
    }
    else {
	my $locstr = "error=yes";
	$locstr .= "&info=$info";
	$locstr .= "&hostname=";
	$locstr .= "&domain=$domain";
	$locstr .= "&ipaddr=$ipaddr";
	$locstr .= "&emaildomain=$emaildomain";
	$locstr .= "&webdomain=$webdomain";
	$locstr .= "&quota=$diskQuota";
	$locstr .= "&maxusers=$maxusers";
	$locstr .= "&shell=$shell";
	$locstr .= "&cgi=$cgi";
	$locstr .= "&ssi=$ssi";
	$locstr .= "&ssl=$ssl";
	$locstr .= "&fpx=$fpx";
	$locstr .= "&ftp=$ftp";
	$locstr .= "&ftpusers=$ftpUsers";
	$locstr .= "&ftpquota=$ftpQuota";
	$locstr .= "&apop=$apop";
	$locstr .= "&bw_setip=$bw_setip";
	$locstr .= "&bwlimit=$bwlimit";
	print "location: $siteDefaultsScriptURL?$locstr\n\n";
    }
}

1;

#
# argument 0 is the object with site info
# argument 1 is the object with bandwidth limit info
#
sub genSiteDefaultsPage {
    my $obj = shift;
    my $bwobj = shift;
    my $info = shift;

    # find default CGI
    my $cgiChecked='';
    if ($obj->get("cgi") eq 'on') { $cgiChecked='CHECKED'; }

    # find default SSI
    my $ssiChecked='';
    if ($obj->get("ssi") eq 'on') { $ssiChecked='CHECKED'; }

    # find default SSL
    my $sslChecked='';
    if ($obj->get("ssl") eq 'on') { $sslChecked='CHECKED'; }

    # find default APOP
    my $apopChecked='';
    if ($obj->get("apop") eq 'on') { $apopChecked='CHECKED'; }

    # find default FTP state
    my $ftpChecked='';
    if ($obj->get("ftp") eq 'on') { $ftpChecked='CHECKED'; }

    # find default FPX state
    my $fpxChecked='';
    if ($obj->get("fpx") eq 'on') { $fpxChecked='CHECKED'; }

    # find default shell account state
    my $shellChecked='';
    if ($obj->get("shell") eq 'on') { $shellChecked='CHECKED'; }

    # find default email at domain state
    my $emaildomainChecked='';
    if ($obj->get("emaildomain") eq 'on') { $emaildomainChecked='CHECKED'; }

    # find default bandwidth setting
    my $bwChecked='';
    if ($obj->get("bwlimit") eq 'on') { $bwChecked='CHECKED'; }

    # find default web at domain state
    my $webdomainChecked='';
    if ($obj->get("webdomain") eq 'on') { $webdomainChecked='CHECKED'; }

    my $ftpQuota = $obj->get("ftpquota");
    my $ftpUsers = $obj->get("ftpusers");
    my $ipaddr = $obj->get("ipaddr");
    my $domain = $obj->get("domain");
    my $maxusers = $obj->get("maxusers");
    my $diskQuota = $obj->get("quota");

    my $bw_setip = $bwobj->get("bwlimit");

    # Read in the template
    open (TEMPLATE, "$Template_dir/siteDefaults.tmpl")
	|| die "Cannot read $Template_dir/siteDefaults.tmpl: $!\n";

    print "content-type: text/html\nCache-Control: no-cache\n\n";

    while( <TEMPLATE> ) {
	s/\[CGICHECKED\]/$cgiChecked/o;
	s/\[SHELLCHECKED\]/$shellChecked/o;
	s/\[SSICHECKED\]/$ssiChecked/o;
	s/\[SSLCHECKED\]/$sslChecked/o;
	s/\[DISKQUOTA\]/$diskQuota/o;
	s/\[FTPCHECKED\]/$ftpChecked/o;
	s/\[APOPCHECKED\]/$apopChecked/o;
	s/\[FTPQUOTA\]/$ftpQuota/o;
	s/\[MAXFTPUSERS\]/$ftpUsers/o;
	s/\[SITEDEFAULTSSCRIPTURL\]/$siteDefaultsScriptURL/o;
	s/\[SITELISTSCRIPTURL\]/$siteListScriptURL/o;
	s/\[USER_INFO\]/$info/o;
	s/\[IPADDR\]/$ipaddr/o;
	s/\[BWLIMIT\]/$bwChecked/o;
	s/\[BW_SETIP\]/$bw_setip/o;
	s/\[MAXUSERS\]/$maxusers/o;
	s/\[FPXCHECKED\]/$fpxChecked/o;
	s/\[DOMAIN\]/$domain/o;
	s/\[EMAILDOMAIN\]/$emaildomainChecked/o;
	s/\[WEBDOMAIN\]/$webdomainChecked/o;

	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/geo;

	print $_;
    }
    close TEMPLATE;
}
