#!/usr/bin/perl

# use strict;

# author: Harris Vaegan-Lloyd
#         Cobalt Neworks
# date: 19-9-99
#
# Copyright 1999-2000, Cobalt Networks, Inc
# All rights reserved

use Cobalt::Product;
use Cobalt::Group;
use Cobalt::Locale;
use Cobalt::Netutil;
use Cobalt::Vacation;
use Cobalt::Vsite;
use Cobalt::Ssl;
use Cobalt::User;
use Cobalt::Util;

use CGI;

use Locale::gettext;

my $forbiddenURL="/.cobalt/error/forbidden.html";
my $siteManageURL='/.cobalt/siteManage/home/?goto=4,0';

# global variables
my $q=new CGI;

# the page that run this script
my $page=$q->param( 'page' );

my $group;
if($ENV{'PARAM_GROUP'}) { 
    $group=$ENV{'PARAM_GROUP'}; 
}
else { 
    $group=$q->param('group'); 
    my $user = $ENV{REMOTE_USER};
    unless (user_authok($user, "group", $group)) {
	print "location: $forbiddenURL\n\n";    
	die "Unauthorised access by $user, not a member of $group.\n";
    }
}

if (!$group && @ARGV) {
    $group=$ARGV[0];
    $page='';
}


my $scriptUrl="/cgi-bin/.cobalt/siteSSL/siteSSL.cgi?group=$group";

my $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); }

if( $page eq 'siteSSL' ) { 
    handleSiteSSLPage(\*STDOUT, $q); 
} else {
    genSiteSSLPage( \*STDOUT, $group );
}

1;


#
# handle redirect
#
sub printRedirect
{
    my ($handle, $q, $url, $location) = @_;
    $location = $url . $location;    

    # handle msie differently. on the other hand, we might want netscape 
    # navigator to show the same behaviour as ie.
    if ($ENV{HTTP_USER_AGENT} =~ /MSIE/) {
	open (TEMPLATE, "$Template_dir/siteSSL_reload.tmpl")
	    || die "Cannot read $Template_dir/siteSSL_reload.tmpl: $!\n";

	print $handle "content-type: text/html\n";
	print $handle "Cache-Control: no-cache\n\n";
	while (<TEMPLATE>) {
	    s/\[SSLURLMSG\]/MSG_get_ui("ssl_redirect_msg", $location)/eo;
	    s/\[str.([^\]]+)\]/MSG_get_ui("$1")/geo;
	    print $handle $_;
	}
	close(TEMPLATE);
    } else {
	print $handle $q->redirect(-location => $location,
				   -target => "_top");
    }
}

#
# handle message for if main site and admserver not rebooted.
#
sub printAdmservReboot
{
    my ($handle, $q, $url, $location) = @_;
    $location = $url . $location;    

    open (TEMPLATE, "$Template_dir/siteSSL_reload.tmpl")
	|| die "Cannot read $Template_dir/siteSSL_reload.tmpl: $!\n";

    print $handle "content-type: text/html\n";
    print $handle "Cache-Control: no-cache\n\n";
    while (<TEMPLATE>) {
	s/\[SSLURLMSG\]/MSG_get_ui("reboot_for_admserv_ssl", $location)/eo;
	s/\[str.([^\]]+)\]/MSG_get_ui("$1")/geo;
	print $handle $_;
    }
    close(TEMPLATE);
}



#
# event handler for add user page
#
sub handleSiteSSLPage {
    my ($handle, $q) = @_;
    my $err=0;
    my $ret;
    my $info='<IMG SRC="/.cobalt/images/bang.gif" ALIGN=CENTER>';
    
    my %subject;
    my $action;
    # read form input
    $subject{'C'} = $q->param('C');
    $subject{'ST'} = $q->param('ST');
    $subject{'L'} = $q->param('L');
    $subject{'O'} = $q->param('O');
    $subject{'OU'} = $q->param('OU');
    $subject{'CN'} = (vsite_get_bygroup($group))[1];
    $subject{'Email'} = $q->param('Email');
    
    my $certificate = $q->param('certificate');
    my $action = $q->param('certopt');
    my $group = $q->param('group');

    
    # Switch on action.
    # Action can be either -
    #		Generating a new self-signed certificate from scratch.
    #		Updating the certificate to one provided by a cert authority.
    #		Averting world war with open-source software.
    if ( $action eq 'gen_cert' ) {
		# Country code must be two uppercase letters.
		$subject{'C'} = uc($subject{'C'});
		if ( length( $subject{'C'} ) != 2 ) {
		    $err = 1;
		    $info .= dgettext('ui',"country_code_mustb2");
		}
		# Check each input to be sure it contains to commans or slashes.
		# This would fubar our parsing code when we get it out of a cert.
		foreach ( keys %subject ) {
		    if ( /[\,\/]/ ) {
			$err = 1;
			$info.=eval("\"".Locale::gettext::dgettext("ui", "ssl_identification_no_comma_slashes")."\"");
		    }
		}
		if (! $err ) {
		    $ret = vsite_ssl_set_identity( $group, 730,
						   $subject{'C'},
						   $subject{'ST'},
						   $subject{'L'},
						   $subject{'O'},
						   $subject{'OU'},
						   $subject{'CN'},
						   $ENV{'REMOTE_USER'}.'@'.$subject{'CN'} );
		    if($ret!~/^2/o) {
			$err=1;
			$info .= substr($ret,4).'<BR>';
		    }
		}
	} elsif ($action eq 'set_new_cert') {
		$ret = ssl_check_valid_cert($certificate);
		if($ret!~/^2/o) {
		    $err=1;
		    $info .= substr($ret,4).'<BR>';
		}
		if (! $err ) {
		    $ret = vsite_ssl_set_cert($group,$certificate);
		    if($ret!~/^2/o) {
				$err=1;
				$info.=substr($ret,4).'<BR>';
		    }
		}
	} elsif ($action eq 'del_cert_and_keys') {
	    $ret = vsite_ssl_clear_all($group);
	    if($ret!~/^2/o) {
		$err=1;
		$info .= substr($ret,4).'<BR>';
	    }
	} elsif ($action eq 'del_cert') {
	    if ($certificate = vsite_ssl_get_cert($group)) {
		$ret = vsite_ssl_clear_cert($group);
		if($ret!~/^2/o) {
		    $err=1;
		    $info .= substr($ret,4).'<BR>';
		}
	    } elsif ($action eq 'gen_cert') {
		$err = 1;
	    }
	}

	# Do no do checking if we just deleted the cert.
	if (($action ne 'del_cert_and_keys') &&
	    ($action ne 'del_cert')) {
	    $ret = vsite_check_cert($group);
	    if($ret!~/^2/o) {
		$err =1;
		$info.=substr($ret,4),'<BR>';
	    }
	}

    my ($obj,$fqdn);
    if ( $err ) {
	genSiteSSLPage($handle, $group,$info);
    } else {
        Cobalt::Http::http_restart(1,1);
	if ($group eq $MainGroup) {
	    # get ssl info and fqdn from database
	    #$obj = Cobalt::Meta->create("type" => "vsite");
	    #$obj->retrieve($group);
	    #$fqdn = $obj->get("hostname") . '.' . $obj->get("domain");
    
	    # We restart admin as well if we've changed the main site certificate.
	    Cobalt::Admserv::admserv_restart(1);
	    # Give apache some time to cope.
	    sleep(2);
	    # Well watson, it would appear that this is activating SSL
	    # for the home site when we're not in by https at the moment.
	      my $url = $ENV{HTTP_HOST}; 
	      if (! defined $ENV{'HTTPS'} &&
		  !($action eq 'del_cert_and_keys')) {
		  $url = 'https://' . $url;
		  printRedirect($handle, $q, $url, $siteManageURL);
		  exit 0;
	      } elsif (defined $ENV{'HTTPS'} && 
 		      (($action eq 'del_cert_and_keys') || ($action eq 'del_cert'))) {
		  $url = 'http://' . $url;
		  printRedirect($handle, $q, $url, $siteManageURL);
		  exit 0;
	      } elsif ($action eq 'set_new_cert') {
		  $url = 'https://' . $url;
		  printAdmservReboot($handle, $q, $url, $siteManageURL);
		  exit 0;
	      }
	      
	  }
	  genSiteSSLPage($handle, $group);
      }
}

#
# argument 0 is the file handle the page goes to
# argument 1 is the group
# argument 2 is the information shown to user
#
sub genSiteSSLPage {
    my ( $handle, $group, $info )=@_;

	my($subject, $issuer, @expires) = vsite_ssl_get_cert_info($group);
	my($cert, $req, $key) = vsite_ssl_get_cert($group);
        my $cert_setting = $cert ? 1 : 0;

	my $HideStart = '<!--';
	my $HideEnd = '-->';

	foreach ( keys %{$subject} ) {
		# If any of the values are different, it must not be a self-signed
		# certificate. Thus display the issuer by clearing out the hide
		# variables;  Check to make sure that there is an issuer though.
		if ( (exists($$issuer{$_})) && 
		     ($$subject{$_} ne $$issuer{$_} )) {
			$HideStart = '';
			$HideEnd = '';
			last;
		}
	}

	my $template = "siteSSL.tmpl";

	if ( $group ne 'home' && ! &vsite_get_ssl($group) ) {
		$template = "siteNoSSL.tmpl";
	}

    # Read in the template
    open (TEMPLATE, "$Template_dir/$template")
	|| die "Cannot read $Template_dir/$template: $!\n";

    print $handle "content-type: text/html\n";
    print $handle "Cache-Control: no-cache\n\n";
    while( <TEMPLATE> ) {
	s/\[SCRIPTURL\]/$scriptUrl/o;
	s/\[USER_INFO\]/$info/o;
	s/\[HIDE_START\]/$HideStart/o;
	s/\[HIDE_END\]/$HideEnd/o;
	s/\[GROUP\]/$group/o;

	s/\[OLDCERT\]/$cert_setting/o;
	s/\[REQUEST\]/$req/o;
	s/\[CERTIFICATE\]/$cert/o;

	s/\[subject.([^\]]+)\]/$$subject{$1}/geo;
	s/\[issuer.([^\]]+)\]/$$issuer{$1}/geo;
	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/geo;
	print $handle $_;
    }
    close TEMPLATE;	    
}
