#!/usr/bin/perl
#
# Copyright 1999-2000, Cobalt Networks, Inc
# All rights reserved

use Cobalt::Product;
use Cobalt::Admserv;
use Cobalt::Locale;
use Cobalt::Netutil;
use Cobalt::Vsite;
use Cobalt::Ftp;
use Cobalt::List;
use Cobalt::Quota;
use Cobalt::Util;
require Cobalt::Meta;

use DB_File;
use Fcntl qw(O_RDWR O_CREAT);
use strict;

my $localeObj=Cobalt::Locale->new();
my $locale=$localeObj->getLocale();

# Update httpd to the current hostname, restarts httpd
vsite_sync();

# to display the host name and IP address on LCD
system( $Lcdshowscript );

# Configure anonymous ftp
if ($Features{'anonFTP'})
{
  # The IP address for $MainGroup is ignored since proftpd.conf contains
  #   no virtual hosting data for $Netif
  # ftp_config configures directories and user accounts automatically
  ftp_config('27.27.27.27',1,0,$MainGroup,$FTPUsersMax,$FTPAnonQuota*=1048576/$BytesPerBlock);
}
                
# restrict access
# be careful to set these in order so that the browser do not ask for password more than once
admserv_set_access( "$AdminPages_dir/admin/", 'admin' );
admserv_set_access( "$AdminPages_dir/alert/", 'admin' );
admserv_set_access( "$AdminPages_dir/cpuUsage/", 'admin' );
admserv_set_access( "$AdminPages_dir/diskUsage/", 'site-admin' );
admserv_set_access( "$AdminPages_dir/network/", 'admin' );
admserv_set_access( "$AdminPages_dir/reboot/", 'admin' );
admserv_set_access( "$AdminPages_dir/services/", 'admin' );
admserv_set_access( "$AdminPages_dir/shutdown/", 'admin' );
admserv_set_access( "$AdminPages_dir/siteAdd/", 'admin' );
admserv_set_access( "$AdminPages_dir/siteDefaults/", 'admin' );
admserv_set_access( "$AdminPages_dir/siteList/", 'admin' );
admserv_set_access( "$AdminPages_dir/sysManage/", 'admin' );
admserv_set_access( "$AdminPages_dir/sysSettings/", 'admin' );
admserv_set_access( "$AdminPages_dir/webUsage/", 'site-admin' );
admserv_set_access( "$AdminPages_dir/wizard/", 'admin' );

admserv_set_access( "$AdminPages_dir/appletData/", 'none' );
admserv_set_access( "$AdminPages_dir/about/", 'none' );
admserv_set_access( "$AdminPages_dir/error/", 'none' );
admserv_set_access( "$AdminPages_dir/htedit/", 'valid' );
admserv_set_access( "$AdminPages_dir/images/", 'none' );
admserv_set_access( "$AdminPages_dir/help/", 'none' );
admserv_set_access( "$AdminPages_dir/info/", 'none' );
admserv_set_access( "$AdminPages_dir/install/", 'admin' );
admserv_set_access( "$AdminPages_dir/javaScript/", 'valid' );
admserv_set_access( "$AdminPages_dir/personal/", 'none' );
admserv_set_access( "$AdminPages_dir/shared/", 'none' );
admserv_set_access( "$AdminPages_dir/siteManage/", 'none' );

admserv_set_access( "$AdminCGIs_dir/admin/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/cpuUsage/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/diagDump/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/disk/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/diskUsage/", 'site-admin' );
admserv_set_access( "$AdminCGIs_dir/dns/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/dnsUsage/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/emailUsage/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/fileUsage/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/info/", 'site-admin' );
admserv_set_access( "$AdminCGIs_dir/install/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/installReplace/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/mailListAdd/", 'site-admin' );
admserv_set_access( "$AdminCGIs_dir/mailListDel/", 'site-admin' );
admserv_set_access( "$AdminCGIs_dir/mailListList/", 'site-admin' );
admserv_set_access( "$AdminCGIs_dir/mailListMod/", 'site-admin' );
admserv_set_access( "$AdminCGIs_dir/memUsage/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/message/", 'valid' );
admserv_set_access( "$AdminCGIs_dir/network/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/networker/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/networkUsage/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/reboot/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/relay/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/services/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/siteAdd/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/siteDefaults/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/siteDel/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/siteDiskUsage/", 'site-admin' );
admserv_set_access( "$AdminCGIs_dir/siteFTP/", 'site-admin' );
admserv_set_access( "$AdminCGIs_dir/siteList/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/siteMod/", 'site-admin' );
admserv_set_access( "$AdminCGIs_dir/siteSSL/", 'site-admin' );
admserv_set_access( "$AdminCGIs_dir/siteUserAdd/", 'site-admin' );
admserv_set_access( "$AdminCGIs_dir/siteUserDefaults/", 'site-admin' );
admserv_set_access( "$AdminCGIs_dir/siteUserDel/", 'site-admin' );
admserv_set_access( "$AdminCGIs_dir/siteUserEmail/", 'site-admin' );
admserv_set_access( "$AdminCGIs_dir/siteUserList/", 'site-admin' );
admserv_set_access( "$AdminCGIs_dir/siteUserMod/", 'site-admin' );
admserv_set_access( "$AdminCGIs_dir/smb/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/snmp/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/snmpUsage/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/sysSettings/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/telnetUsage/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/time/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/tools/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/totUsage/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/webLog/", 'site-admin' );
admserv_set_access( "$AdminCGIs_dir/webUsage/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/wizard/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/userDefaults/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/arkeia/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/alert/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/alertSignal/", 'admin' );
admserv_set_access( "$AdminCGIs_dir/backup/", 'valid' );
admserv_set_access( "$AdminCGIs_dir/email/", 'valid' );
admserv_set_access( "$AdminCGIs_dir/profileMod/", 'valid' );
admserv_set_access( "$AdminCGIs_dir/userMod/", 'valid' );
admserv_set_access( "$AdminCGIs_dir/userUsage/", 'valid' );

admserv_set_access( "$UserCGIs_dir/search/private/", 'valid' );

# default object
my $obj;
$obj = Cobalt::Meta->new("type"         => "vsite",
			 "name"         => "default",
			 "quota"        => 200,
			 "shell"        => "off",
			 "ftp"          => "off",
			 "ftpquota"     => 20,
			 "ftpusers"     => 10,
			 "gid"          => 9999,
			 "maxusers"     => 25,
			 "cgi"          => "off",
			 "ssi"          => "off",
			 "ssl"          => "off",
			 "fpx"          => "off",
			 "suspend"      => "off",
			 "bwlimit"      => "off",
			 "apop"         => "off",
			 "emaildomain"  => "off",
			 "webdomain"    => "off",
			 "ipaddr"       => "",
			 "domain"       => "",
			 "hostname"     => "",
			 "fqdn"         => "",
			 "user_quota"   => 50,
			 "user_fpx"     => "off",
			 "user_shell"   => "off",
			 "user_apop"    => "off",
			 "user_namegen" => 0,
			 "userlist_sort" => "name",
			 "userlist_range" => 15);
$obj->save;

# get host/domain/ip for main site for database
my $ipaddr = (Cobalt::Network::net_get_ipaddr())[0];
my $hostname = Cobalt::Network::net_get_hostname();
my $fqdn = Cobalt::Netutil::netutil_get_fulldomain();
$fqdn =~ /^$hostname\.(.*)/;
my $domain = $1;
my $gid = (getgrnam($MainGroup))[2];

# initialize database for home site
$obj = Cobalt::Meta->new("type"         => "vsite",
			 "name"         => "home",
			 "quota"        => 1000,
			 "shell"        => "off",
			 "ftp"          => "off",
			 "ftpquota"     => 20,
			 "ftpusers"     => 10,
			 "gid"          => $gid,
			 "maxusers"     => 100,
			 "cgi"          => "off",
			 "ssi"          => "off",
			 "ssl"          => "off",
			 "fpx"          => "off",
			 "suspend"      => "off",
			 "bwlimit"      => "off",
			 "apop"         => "off",
			 "emaildomain"  => "off",
			 "webdomain"    => "off",
			 "ipaddr"       => "$ipaddr",
			 "domain"       => "$domain",
			 "hostname"     => "$hostname",
			 "fqdn"         => "$fqdn",
			 "user_quota"   => 50,
			 "user_fpx"     => "off",
			 "user_shell"   => "off",
			 "user_apop"    => "off",
			 "user_namegen" => 0,
			 "userlist_sort" => "name",
			 "userlist_range" => 15);
$obj->save;

$obj = Cobalt::Meta->new("type"        => "users",
			 "name"        => "admin",
			 "fullname"    => "Administrator",
			 "aliases"     => "",
			 "altname"     => "",
			 "vsite"       => "",
			 "vacationmsg" => "",
			 "uid"         => 110,
			 "quota"       => 0,
			 "admin"       => "on",
			 "apop"        => "off",
			 "suspend"     => "off",
			 "forward"     => "off",
			 "vacation"    => "off",
			 "shell"       => "off",
			 "fpx"         => "off");
$obj->save;

$obj = Cobalt::Meta->new("type"      => "bw",
			 "name"      => "default",
			 "method"    => "ip",
			 "bwlimit"   => "",
			 "protocol"  => "tcp",
			 "port"      => "",
			 "direction" => "w");
$obj->save;

# set quota for main site
Cobalt::Quota::setqlim("$Htgroup_dir/$MainGroup",$gid,1024000,1024001,1);

# create a virtual list server for the main site
list_create_virtual ($MainGroup, $fqdn);

# restrict userAdd access according to access mode
admserv_set_access( "$AdminCGIs_dir/userAdd", 'admin' );
admserv_set_access( "$AdminPages_dir/userAdd", 'admin' );
admserv_set_access( "$UserPages_dir/userAddU", 'admin' );

# clean up wizard pages
system( "/bin/rm -f $Htgroup_dir/$MainGroup/$Webdir/index.html &>/dev/null" );
system( "/bin/rm -f $AdminServer_dir/html/index.html &>/dev/null" );
system( "/bin/rm -f $AdminServer_dir/html/index.html.en &>/dev/null" );

# make main page
system( "/bin/cp -f $Group_skel/home/$locale/$Webdir/index.html $Htgroup_dir/$MainGroup/$Webdir/" );

# owned by group
my $gid=( getgrnam( $MainGroup ) )[ 2 ];
system( "/bin/chown $WWWuid:$gid $Htgroup_dir/$MainGroup/$Webdir/index.html" );
system( "/bin/chmod 0664 $Htgroup_dir/$MainGroup/$Webdir/index.html" );

# rotate the main sites web access log
Cobalt::Vsite::vsite_set_logrotate($MainGroup,30);

# re-make admin's home page, with the correct fqdn
if (-f "$User_skel/$locale/$Webdir/index.html") {
    open (SRC,"$User_skel/$locale/$Webdir/index.html");
    open (DEST,">$Htgroup_dir/$MainGroup/users/$Adminuser/$Webdir/index.html");
    while (<SRC>)
    {
	if ( /\[/o )
	{
	    s/\[DOMAIN\]/$fqdn/go;
	    s/\[PATH\]/index.html/o;
	    s/\[USER\]/MSG_get("DefaultAdminFullName")/eo;
	    s/\[USERNAME\]/$Adminuser/go;
	}
	print DEST;
    }
    close DEST;
    close SRC;
chown( (getpwnam($Adminuser))[2], (getgrnam($MainGroup))[2],
"$Htgroup_dir/$MainGroup/users/$Adminuser/$Webdir/index.html");
chmod(0664,"$Htgroup_dir/$MainGroup/users/$Adminuser/$Webdir/index.html");
}

###
# ManageRaQ wants and needs if enabled
if ($Features{'mgmt'}) {
	my ($base_page) .=<<EOF;
<HTML>
<HEAD>
<META HTTP-EQUIV="Refresh" CONTENT="0;URL=https://$ipaddr:449/">
</HEAD>
</HTML>
EOF

	open(OUT,">$Lockdir/index.html");
	print OUT $base_page;
	close OUT;
	rename("$Lockdir/index.html","$AdminPages_dir/about/index.html")
		|| return Cobalt::Network::net_abort(MSG_error("4205",
		"$Lockdir/index.html"));
	system("cp -f $AdminPages_dir/about/index.html $Htgroup_dir/$MainGroup/$Webdir/index.html");
}
# Permission? Ah, let the admin have it.
chown( (getpwnam($Adminuser))[2], (getgrnam($MainGroup))[2],
   "$Htgroup_dir/$MainGroup/$Webdir/index.html");
chmod(0664,"$Htgroup_dir/$MainGroup/$Webdir/index.html");

# remove swatch's lockfile
unlink("/etc/locks/LCK..swatch");

my $rmd5 = `md5sum /var/run/random-seed`;
my $p = 'c' . substr($rmd5, 0, 15);
Cobalt::Meta::setpw("cobalt", "5583", "admin", $p);
sleep(2);

if( $Features{ 'cobaltHomePage' } ) {
# redirect to cobalt homepage
print "location: /.user/cobalt/\n\n";
}

exit 0;

