// Author: Kevin K.M. Chiu
// Version: 1.0
// Date: 7-10-98
// Copyright 1998-1999, Cobalt Networks.  All rights reserved.

function checkNetAddr( element, errMsg ) {
    for( var i=0; i<element.value.length; i++ ) {
	var ch=element.value.charAt( i );
	if( !( ( ch>='0' && ch<='9' ) || ch=='.' ) )
	    return checkDomainName( element, errMsg );
    }

    return checkIPAddr( element, errMsg );
}

function checkIPAddr( element, errMsg ) {
    if( element.value.length>0 && !isIPAddrValid( element.value ) ) {
	element.focus();
	element.select();
	top.showInfo( errMsg );

	return false;
    }
    return true;
}

function checkDomainName( element, errMsg ) {
    if( element.value.length>0 && !isDomainNameValid( element.value ) ) {
	element.focus();
	element.select();
	top.showInfo( errMsg );

	return false;
    }
    return true;
}

function isIPAddrValid( ipAddr ) {
    var dotCount=0;

    // only digits and '.' allowed
    for( i=0; i<ipAddr.length; i++ ) {
	var ch=ipAddr.charAt( i );
	if( ( ch<'0' || ch>'9' ) && ch!='.' ) return false;

	if( ch=='.' ) dotCount++;
    }

    // only 3 dots
    if( dotCount!=3 ) return false;

    var numbers=new Object();
    var lastPtr=0;
    var ptr;
    for( i=0; i<3; i++ ) {
	ptr=ipAddr.indexOf( '.', lastPtr );
	numbers[ i ]=ipAddr.substring( lastPtr, ptr );
	lastPtr=ptr+1;
    }
    numbers[ 3 ]=ipAddr.substring( lastPtr );

    for( i=0; i<4; i++ ) {
	if( numbers[ i ].length==0 ) return false;
	if( numbers[ i ]>255 ) return false;
    }

    // no 255.255.255.255
    if( numbers[ 0 ]==0 && numbers[ 1 ]==0 && numbers[ 2 ]==0 && numbers[ 3 ]==0 ) return false;
    if( numbers[ 0 ]==255 && numbers[ 1 ]==255 && numbers[ 2 ]==255 && numbers[ 3 ]==255 ) return false;

    return true;
}

function isDomainNameValid( domainName ) {
    if( domainName.charAt( domainName.length-1 )=='.' ) return false;

    var letterExist=false;
    for( var i=0; i<domainName.length; i++ ) {
	var ch=domainName.charAt( i );
	if( ch>='a' && ch<='z' ) { letterExist=true; break; }
    }
    if( !letterExist ) return false;

    var ptr=0;
    while( ptr<domainName.length ) {
	var dotPtr=domainName.indexOf( '.', ptr );
	if( dotPtr==-1 ) dotPtr=domainName.length;

	// JPNIC worships 1 character subdomain
	if( dotPtr==ptr ) return false;
//	if( dotPtr==ptr+1 || dotPtr==ptr ) return false;

	var subdomain=domainName.substring( ptr, dotPtr );
	var ch=subdomain.charAt( 0 );

	if( !( ch>='a' && ch<='z' ) && !( ch>='0' && ch<='9' ) ) return false;
	if( subdomain.indexOf( '--' )!=-1 ) return false;
	for( i=0; i<subdomain.length; i++ ) {
	    var ch=subdomain.charAt( i );
	    if( !( ( ch>='a' && ch<='z' ) || ( ch>='0' && ch<='9' ) || ch=='-' ) ) return false;
	}

	ptr=dotPtr+1;
    }

    return true;
}